@@ -186,6 +186,16 @@ public:
     * @returns the current album
     */
     Album*    currentAlbum() const;
+
+    /**
+     * @returns the current PAlbum or null if no one is selected
+     */
+    PAlbum* currentPAlbum() const;
+
+    /**
+     * @returns the current TAlbum or null if no one is selected
+     */
+    TAlbum* currentTAlbum() const;
     //@}
 
     /** @name Finding Albums
@@ -238,7 +248,7 @@ public:
 
     /**
      * @return a SAlbum with given name, or 0 if not found
-     * @param tagPath the name of the search
+     * @param name the name of the search
      */
     SAlbum*   findSAlbum(const QString& name) const;
     //@}
@@ -260,7 +270,6 @@ public:
      * @param name    the name of the new album
      * @param caption the caption for the new album
      * @param date    the date for the new album
-     * @param collection the collection for the new album
      * @param errMsg  this will contain the error message describing why the
      * operation failed
      */
@@ -281,6 +290,8 @@ public:
     /**
      * Overloaded method. Here you can supply a collection location (which
      * must be available).
+     *
+     * @param location the collection for the new album
      */
     PAlbum* createPAlbum(const CollectionLocation& location, const QString& name,
                          const QString& caption, const QDate& date,
@@ -338,7 +349,7 @@ public:
     /**
      * A list of tag paths is supplied.
      * If no corresponding TAlbum exists, a new one will be created.
-     * @param tagPath A list of tag paths
+     * @param tagPaths A list of tag paths
      * @returns A list of all TAlbums for the list (already existing or newly created)
     */
     AlbumList findOrCreateTAlbums(const QStringList& tagPaths);
@@ -401,6 +412,7 @@ public:
     /**
      * @return A list with the tag paths for a list of tag IDs.
      * @param tagIDs list of tag album IDs
+     * @param leadingSlash if <code>true</code> return tags with a leading slash
      */
     QStringList tagPaths(const QList<int>& tagIDs, bool leadingSlash=true) const;
 
@@ -433,9 +445,9 @@ public:
      * \note the signalAlbumAdded will be fired before this function returns. Its
      * recommended to connect to that signal to get notification of new album added
      * @return the newly created SAlbum or an existing SAlbum with same name
-     * @param url    the url of the album
-     * @param simple indicates whether the Search album is of simple type or
-     * extended type
+     * @param name  name for the new search
+     * @param type  the type of the search
+     * @param query search query to use
      */
     SAlbum* createSAlbum(const QString& name, DatabaseSearch::Type type, const QString& query);
 
@@ -443,7 +455,7 @@ public:
      * Update the url for a SAlbum
      * @return true if the operation succeeds, false otherwise
      * @param album the album to update
-     * @param query the new query data of the album
+     * @param changedQuery the new query data of the album
      * @param changedName a new name, or null to keep the current name
      * @param type a new type, or UndefinedType to keep the current type
      */
@@ -460,16 +472,55 @@ public:
     bool deleteSAlbum(SAlbum* album);
     //@}
 
+    /** @name Accessors to counting maps
+     */
+    //@{
+
+    /**
+     * Returns the latest count for PAlbums as also emitted via
+     * signalPAlbumsDirty.
+     *
+     * @return count map for PAlbums
+     */
+    QMap<int, int> getPAlbumsCount() const;
+
+    /**
+     * Returns the latest count for TAlbums as also emitted via
+     * signalTAlbumsDirty.
+     *
+     * @return count map for TAlbums
+     */
+    QMap<int, int> getTAlbumsCount() const;
+
+    /**
+     * Returns the latest count for DAlbums as also emitted via
+     * signalDAlbumsDirty.
+     *
+     * @return count map for DAlbums
+     */
+    QMap<YearMonth, int> getDAlbumsCount() const;
+
+    /**
+     * Returns if the given album is currently being moved, that is,
+     * if this album is in between signalAlbumAboutToBeMoved and
+     * signalAlbumMoved. In this case, you can preserve state of such an album
+     * because the object is guaranteed not to be deleted, even if
+     * signalAlbumAboutToBeDeleted is emitted.
+     */
+    bool isMovingAlbum(Album* album) const;
+
+    //@}
+
 Q_SIGNALS:
 
     /// An album is about to be added to the given parent (0 if album is root)
     /// after the item given by prev (prev is 0 if parent has no children yet)
     void signalAlbumAboutToBeAdded(Album *album, Album *parent, Album *prev);
-    /// The album has been added
+    /// The album has been added.
     void signalAlbumAdded(Album* album);
-    /// The album is about to be deleted, but is still fully valid
+    /// The album is about to be deleted, but is still fully valid.
     void signalAlbumAboutToBeDeleted(Album *album);
-    /// The album is deleted, but the object can still be accessed
+    /// The album is deleted, but the object can still be accessed.
     void signalAlbumDeleted(Album* album);
     /// The album is deleted, the object can no longer be accessed.
     /// For identification purposes, the former album pointer is passed.
@@ -481,7 +532,12 @@ Q_SIGNALS:
     void signalAlbumIconChanged(Album* album);
     void signalAlbumRenamed(Album* album);
     void signalSearchUpdated(SAlbum *album);
-    void signalTAlbumMoved(TAlbum* album, TAlbum* newParent);
+    /// Indicates that an album is about to be moved. Signals for deleting and adding will be
+    /// sent afterwards, but the album object is guaranteed not to be deleted until after signalAlbumMoved.
+    void signalAlbumAboutToBeMoved(Album* album);
+    /// Emitted when the album is moved to its new parent. After signalAlbumAboutToBeMoved,
+    /// all four signals for first deleting and then adding will have been sent.
+    void signalAlbumMoved(Album* album);
     void signalPAlbumsDirty(const QMap<int, int>&);
     void signalTAlbumsDirty(const QMap<int, int>&);
     void signalDAlbumsDirty(const QMap<YearMonth, int>&);
@@ -511,20 +567,20 @@ private Q_SLOTS:
     /**
      * Scan albums directly from database and creates new PAlbums
      * It only creates those PAlbums which haven't already been
-     * created
+     * created.
      */
     void scanPAlbums();
     void updateChangedPAlbums();
     /**
      * Scan tags directly from database and creates new TAlbums
      * It only creates those TAlbums which haven't already been
-     * created
+     * created.
      */
     void scanTAlbums();
     /**
      * Scan searches directly from database and creates new SAlbums
      * It only creates those SAlbums which haven't already been
-     * created
+     * created.
      */
     void scanSAlbums();
     /**
@@ -542,6 +598,16 @@ private:
     AlbumManager();
     ~AlbumManager();
 
+    /**
+     * Checks whether an Album has a direct child with the given name.
+     *
+     * @param parent album to check children for
+     * @param title title to search for
+     * @return <code>true</code> if there is a child with name, else
+     *         <code>false</code>
+     */
+    bool hasDirectChildAlbumWithTitle(Album *parent, const QString &title);
+
     void insertPAlbum(PAlbum *album, PAlbum *parent);
     void removePAlbum(PAlbum *album);
     void insertTAlbum(TAlbum *album, TAlbum *parent);
