@@ -23,7 +23,7 @@
 
 #include "upgradedb_sqlite2tosqlite3.h"
 
-// Qt includes.
+// Qt includes
 
 #include <QMap>
 #include <QPair>
@@ -32,13 +32,13 @@
 #include <QFileInfo>
 #include <QList>
 
-// KDE includes.
+// KDE includes
 
 #include <kdebug.h>
 #include <kstandarddirs.h>
 #include <kio/global.h>
 
-// Local includes.
+// Local includes
 
 #include "albumdb.h"
 #include "databaseaccess.h"
@@ -97,14 +97,14 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
 
     #ifdef NFS_HACK
     newDB = locateLocal("appdata", KIO::encodeFileName(QDir::cleanPath(newDB)));
-    kDebug(50003) << "NFS: " << newDB << endl;
+    kDebug() << "NFS: " << newDB;
     #endif
 
     AlbumDB db3;
     access.db()->setDBPath(newDB);
     if (!access.db()->isValid())
     {
-        kWarning(50003) << "Failed to open new Album Database" << endl;
+        kWarning() << "Failed to open new Album Database";
         return false;
     }
     */
@@ -119,7 +119,7 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
     /*
     #ifdef NFS_HACK
     dbPath = locateLocal("appdata", KIO::encodeFileName(QDir::cleanPath(dbPath)));
-    kDebug(50003) << "From NFS: " << dbPath << endl;
+    kDebug() << "From NFS: " << dbPath;
     #endif
     */
 
@@ -127,7 +127,7 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
 
     if (!fi.exists())
     {
-        kDebug(50003) << "No old database present. Not upgrading" << endl;
+        kDebug() << "No old database present. Not upgrading";
         access.db()->setSetting("UpgradedFromSqlite2", "yes");
         return true;
     }
@@ -136,7 +136,7 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
     db2.setDBPath( dbPath );
     if (!db2.isValid())
     {
-        kDebug(50003) << "Failed to initialize Old Album Database" << endl;
+        kDebug() << "Failed to initialize Old Album Database";
         return false;
     }
 
@@ -163,7 +163,7 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
     AlbumMap albumMap;
 
     access.backend()->beginTransaction();
-    for (QStringList::iterator it=values.begin(); it!=values.end();)
+    for (QStringList::const_iterator it=values.constBegin(); it!=values.constEnd();)
     {
         _Album album;
 
@@ -202,7 +202,7 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
     TagList tagList;
 
     access.backend()->beginTransaction();
-    for (QStringList::iterator it=values.begin(); it!=values.end();)
+    for (QStringList::const_iterator it=values.constBegin(); it!=values.constEnd();)
     {
         _Tag tag;
 
@@ -230,7 +230,7 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
                 &values);
 
     access.backend()->beginTransaction();
-    for (QStringList::iterator it=values.begin(); it!=values.end();)
+    for (QStringList::const_iterator it=values.constBegin(); it!=values.constEnd();)
     {
         int dirid   = (*it).toInt();
         ++it;
@@ -249,7 +249,7 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
     db2.execSql("SELECT dirid, name, tagid FROM ImageTags;",
                 &values);
     access.backend()->beginTransaction();
-    for (QStringList::iterator it=values.begin(); it!=values.end();)
+    for (QStringList::const_iterator it=values.constBegin(); it!=values.constEnd();)
     {
         int dirid = (*it).toInt();
         ++it;
@@ -270,7 +270,7 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
     // update album icons -------------------------------------------------
 
     access.backend()->beginTransaction();
-    for (AlbumList::iterator it = albumList.begin(); it != albumList.end();
+    for (AlbumList::const_iterator it = albumList.constBegin(); it != albumList.constEnd();
          ++it)
     {
         _Album album = *it;
@@ -288,7 +288,7 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
     // -- update tag icons ---------------------------------------------------
 
     access.backend()->beginTransaction();
-    for (TagList::iterator it = tagList.begin(); it != tagList.end(); ++it)
+    for (TagList::const_iterator it = tagList.constBegin(); it != tagList.constEnd(); ++it)
     {
         _Tag tag = *it;
 
@@ -312,8 +312,8 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
         AlbumMap::iterator it1 = albumMap.find(url);
         if (it1 == albumMap.end())
         {
-            kDebug(50003) << "Could not find album with url: " << url << endl;
-            kDebug(50003) << "Most likely an external directory. Rejecting." << endl;
+            kDebug() << "Could not find album with url: " << url;
+            kDebug() << "Most likely an external directory. Rejecting.";
             continue;
         }
 
@@ -333,15 +333,15 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
     // -- update setting entry ------------------------------------------
     access.db()->setSetting("UpgradedFromSqlite2", "yes");
 
-    kDebug(50003) << "Successfully upgraded database to sqlite3 " << endl;
+    kDebug() << "Successfully upgraded database to sqlite3 ";
 
     // -- Check for db consistency ----------------------------------------
 
     /*
-    kDebug(50003) << "Checking database consistency" << endl;
+    kDebug() << "Checking database consistency";
 
 
-    kDebug(50003) << "Checking Albums.................." << endl;
+    kDebug() << "Checking Albums..................";
     values.clear();
     db2.execSql("SELECT id, url, date, caption, collection FROM Albums;", &values);
     for (QStringList::iterator it = values.begin(); it != values.end();)
@@ -372,17 +372,17 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
                 .arg(access.backend()->escapeString(album.collection)), &list, false);
         if (list.size() != 1)
         {
-            kError(50003) << "Failed" << endl;
-            kWarning(50003) << "" << endl;
-            kWarning(50003) << "Consistency check failed for Album: "
-                       << album.url << endl;
+            kError() << "Failed";
+            kWarning() << "";
+            kWarning() << "Consistency check failed for Album: "
+                       << album.url;
             return false;
         }
     }
-    kDebug(50003) << " (" << values.count()/5 << " Albums) "  << "OK" << endl;
+    kDebug() << " (" << values.count()/5 << " Albums) "  << "OK";
 
 
-    kDebug(50003) << "Checking Tags...................." << endl;
+    kDebug() << "Checking Tags....................";
     values.clear();
     db2.execSql("SELECT id, pid, name FROM Tags;", &values);
     for (QStringList::iterator it = values.begin(); it != values.end();)
@@ -405,17 +405,17 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
         &list, false);
         if (list.size() != 1)
         {
-            kError(50003) << "Failed" << endl;
-            kWarning(50003) << "" << endl;
-            kWarning(50003) << "Consistency check failed for Tag: "
-                       << name << endl;
+            kError() << "Failed";
+            kWarning() << "";
+            kWarning() << "Consistency check failed for Tag: "
+                       << name;
             return false;
         }
     }
-    kDebug(50003) << " (" << values.count()/3 << " Tags) "  << "OK" << endl;
+    kDebug() << " (" << values.count()/3 << " Tags) "  << "OK";
 
 
-    kDebug(50003) << "Checking Images.................." << endl;
+    kDebug() << "Checking Images..................";
     values.clear();
     db2.execSql("SELECT Albums.url, Images.name, Images.caption "
             "FROM Images, Albums WHERE Albums.id=Images.dirid;", &values);
@@ -440,17 +440,17 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
         &list, false);
         if (list.size() != 1)
         {
-            kError(50003) << "Failed" << endl;
-            kWarning(50003) << "" << endl;
-            kWarning(50003) << "Consistency check failed for Image: "
-                       << url << ", " << name << ", " << caption  << endl;
+            kError() << "Failed";
+            kWarning() << "";
+            kWarning() << "Consistency check failed for Image: "
+                       << url << ", " << name << ", " << caption;
             return false;
         }
     }
-    kDebug(50003) << " (" << values.count()/3 << " Images) " << "OK" << endl;
+    kDebug() << " (" << values.count()/3 << " Images) " << "OK";
 
 
-    kDebug(50003) << "Checking ImageTags..............." << endl;
+    kDebug() << "Checking ImageTags...............";
     values.clear();
     db2.execSql("SELECT Albums.url, ImageTags.name, ImageTags.tagid "
             "FROM ImageTags, Albums WHERE \n "
@@ -477,16 +477,16 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
         &list, false);
         if (list.size() != 1)
         {
-            kError(50003) << "Failed" << endl;
-            kWarning(50003) << "" << endl;
-            kWarning(50003) << "Consistency check failed for ImageTag: "
-                       << url << ", " << name << ", " << tagid << endl;
+            kError() << "Failed";
+            kWarning() << "";
+            kWarning() << "Consistency check failed for ImageTag: "
+                       << url << ", " << name << ", " << tagid;
             return false;
         }
     }
-    kDebug(50003) << " (" << values.count()/3 << " ImageTags) " << "OK" << endl;
+    kDebug() << " (" << values.count()/3 << " ImageTags) " << "OK";
 
-    kDebug(50003) << "Checking Album icons ..............." << endl;
+    kDebug() << "Checking Album icons ...............";
     values.clear();
     db2.execSql("SELECT url, icon FROM Albums;", &values);
     for (QStringList::iterator it = values.begin(); it != values.end();)
@@ -509,18 +509,18 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
 
         if (list.size() != 1)
         {
-            kError(50003) << "Failed" << endl;
-            kWarning(50003) << "" << endl;
-            kWarning(50003) << "Consistency check failed for Album Icon: "
-                       << url << ", " << icon << endl;
+            kError() << "Failed";
+            kWarning() << "";
+            kWarning() << "Consistency check failed for Album Icon: "
+                       << url << ", " << icon;
 
             return false;
         }
     }
-    kDebug(50003) << " (" << values.count()/2 << " Album Icons) " << "OK" << endl;
+    kDebug() << " (" << values.count()/2 << " Album Icons) " << "OK";
 
 
-    kDebug(50003) << "Checking Tag icons ..............." << endl;
+    kDebug() << "Checking Tag icons ...............";
     values.clear();
     db2.execSql("SELECT id, icon FROM Tags;", &values);
     for (QStringList::iterator it = values.begin(); it != values.end();)
@@ -544,10 +544,10 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
 
             if (list.size() != 1)
             {
-                kError(50003) << "Failed" << endl;
-                kWarning(50003) << "" << endl;
-                kWarning(50003) << "Consistency check failed for Tag Icon: "
-                           << id << ", " << icon << endl;
+                kError() << "Failed";
+                kWarning() << "";
+                kWarning() << "Consistency check failed for Tag Icon: "
+                           << id << ", " << icon;
 
                 return false;
             }
@@ -567,8 +567,8 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
                     .arg(access.backend()->escapeString(url)), &list);
             if (list.isEmpty())
             {
-                kWarning(50003) << "Tag icon not in Album Library Path, Rejecting " << endl;
-                kWarning(50003) << "(" << icon << ")" << endl;
+                kWarning() << "Tag icon not in Album Library Path, Rejecting ";
+                kWarning() << "(" << icon << ")";
                 continue;
             }
 
@@ -583,20 +583,20 @@ bool upgradeDB_Sqlite2ToSqlite3(DatabaseAccess &access, const QString& sql2DBPat
                     .arg(id), &list);
             if (list.size() != 1)
             {
-                kError(50003) << "Failed." << endl;
-                kWarning(50003) << "" << endl;
-                kWarning(50003) << "Consistency check failed for Tag Icon: "
-                           << id << ", " << icon << endl;
+                kError() << "Failed.";
+                kWarning() << "";
+                kWarning() << "Consistency check failed for Tag Icon: "
+                           << id << ", " << icon;
 
                 return false;
             }
 
         }
     }
-    kDebug(50003) << " (" << values.count()/2 << " Tag Icons) " << "OK" << endl;
+    kDebug() << " (" << values.count()/2 << " Tag Icons) " << "OK";
 
-    kDebug(50003) << "" << endl;
-    kDebug(50003) << "All Tests: A-OK" << endl;
+    kDebug() << "";
+    kDebug() << "All Tests: A-OK";
     */
 
     return true;
