@@ -7,8 +7,8 @@
  * Description : A digital camera RAW files loader for DImg
  *               framework using an external dcraw instance.
  *
- * Copyright (C) 2005-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2005-2008 by Marcel Wiesweg <marcel.wiesweg@gmx.de>
+ * Copyright (C) 2005-2010 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2005-2010 by Marcel Wiesweg <marcel.wiesweg@gmx.de>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -23,7 +23,6 @@
  *
  * ============================================================ */
 
-
 #include "rawloader.moc"
 
 // C++ includes
@@ -45,10 +44,10 @@
 #include "dimgloaderobserver.h"
 #include "icctransform.h"
 #include "imagehistogram.h"
-#include "imagecurves.h"
 #include "imagelevels.h"
-#include "bcgmodifier.h"
-#include "whitebalance.h"
+#include "curvesfilter.h"
+#include "bcgfilter.h"
+#include "wbfilter.h"
 #include "globals.h"
 
 namespace Digikam
@@ -62,7 +61,7 @@ RAWLoader::RAWLoader(DImg* image, DRawDecoding rawDecodingSettings)
     m_observer            = 0;
 }
 
-bool RAWLoader::load(const QString& filePath, DImgLoaderObserver *observer)
+bool RAWLoader::load(const QString& filePath, DImgLoaderObserver* observer)
 {
     m_observer = observer;
 
@@ -130,21 +129,8 @@ void RAWLoader::setWaitingDataProgress(double value)
         m_observer->progressInfo(m_image, value);
 }
 
-#if KDCRAW_VERSION < 0x000400
-bool RAWLoader::checkToCancelReceivingData()
-{
-    return (m_observer ? m_observer->isShuttingDown() : false);
-}
-
-void RAWLoader::setReceivingDataProgress(double value)
-{
-    if (m_observer)
-        m_observer->progressInfo(m_image, value);
-}
-#endif
-
 bool RAWLoader::loadedFromDcraw(QByteArray data, int width, int height, int rgbmax,
-                                DImgLoaderObserver *observer)
+                                DImgLoaderObserver* observer)
 {
     int checkpoint = 0;
 
@@ -178,15 +164,9 @@ bool RAWLoader::loadedFromDcraw(QByteArray data, int width, int height, int rgbm
             {
                 if (QSysInfo::ByteOrder == QSysInfo::LittleEndian)     // Intel
                 {
-                    #if KDCRAW_VERSION < 0x000400
-                    dst[0] = (unsigned short)((src[4]*256 + src[5]) * fac);      // Blue
-                    dst[1] = (unsigned short)((src[2]*256 + src[3]) * fac);      // Green
-                    dst[2] = (unsigned short)((src[0]*256 + src[1]) * fac);      // Red
-                    #else
                     dst[0] = (unsigned short)((src[5]*256 + src[4]) * fac);      // Blue
                     dst[1] = (unsigned short)((src[3]*256 + src[2]) * fac);      // Green
                     dst[2] = (unsigned short)((src[1]*256 + src[0]) * fac);      // Red
-                    #endif
                 }
                 else
                 {
@@ -201,63 +181,6 @@ bool RAWLoader::loadedFromDcraw(QByteArray data, int width, int height, int rgbm
             }
         }
 
-#if KDCRAW_VERSION < 0x000400
-        // ----------------------------------------------------------
-
-        // Special case: if Color Management is not used here, output color space is in sRGB* color space
-        // RAW decoded image is a linear-histogram image with 16 bits color depth.
-        // No auto white balance and no gamma adjustments are performed. Image is a black hole.
-        // We need to reproduce all dcraw 8 bits color depth adjustments here.
-
-        if (m_rawDecodingSettings.outputColorSpace != DRawDecoding::RAWCOLOR)
-        {
-            ImageHistogram histogram(image, width, height, true);
-            histogram.calculate();
-
-            int perc, val, total;
-            float white=0.0, r, gamma=2.222222;
-            unsigned short lut[65536];
-
-            // Search 99th percentile white level.
-
-            perc = (int)(width * height * 0.01);
-            kDebug() << "White Level: " << perc;
-            for (int c = 1 ; c < 4 ; ++c)
-            {
-                total = 0;
-                for (val = 65535 ; val > 256 ; --val)
-                    if ((total += (int)histogram.getValue(c, val)) > perc)
-                        break;
-
-                if (white < val) white = (float)val;
-            }
-
-            white *= 1.0 / m_rawDecodingSettings.brightness;
-
-            kDebug() << "White Point: " << white;
-
-            // Compute the Gamma lut accordingly.
-
-            for (int i=0; i < 65536; ++i)
-            {
-                r = i / white;
-                val = (int)(65536.0 * (r <= 0.018 ? r*4.5 : pow(r, 1.0/gamma) * 1.099-0.099));
-                if (val > 65535) val = 65535;
-                lut[i] = val;
-            }
-
-            //  Apply Gamma lut to the whole image.
-
-            unsigned short *im = (unsigned short *)image;
-            for (int i = 0; i < width*height; ++i)
-            {
-                im[0] = lut[im[0]];      // Blue
-                im[1] = lut[im[1]];      // Green
-                im[2] = lut[im[2]];      // Red
-                im += 4;
-            }
-        }
-#endif
         // ----------------------------------------------------------
 
         imageData() = (uchar *)image;
@@ -352,13 +275,14 @@ bool RAWLoader::loadedFromDcraw(QByteArray data, int width, int height, int rgbm
     imageWidth()  = width;
     imageHeight() = height;
     imageSetAttribute("format", "RAW");
+    imageSetAttribute("rawDecodingSettings", QVariant::fromValue(m_customRawSettings));
     imageSetAttribute("originalColorModel", DImg::COLORMODELRAW);
     imageSetAttribute("originalBitDepth", 16);
 
     return true;
 }
 
-void RAWLoader::postProcess(DImgLoaderObserver *observer)
+void RAWLoader::postProcess(DImgLoaderObserver* observer)
 {
     // emulate LibRaw custom output profile
     if (!m_customOutputProfile.isNull())
@@ -377,40 +301,46 @@ void RAWLoader::postProcess(DImgLoaderObserver *observer)
 
     if (m_customRawSettings.exposureComp != 0.0 || m_customRawSettings.saturation != 1.0)
     {
-        WhiteBalance wb(m_rawDecodingSettings.sixteenBitsImage);
-        wb.whiteBalance(imageData(), imageWidth(), imageHeight(), m_rawDecodingSettings.sixteenBitsImage,
-                        0.0,                                // black
-                        m_customRawSettings.exposureComp,   // exposure
-                        6500.0,                             // temperature (neutral)
-                        1.0,                                // green
-                        0.5,                                // dark
-                        1.0,                                // gamma
-                        m_customRawSettings.saturation);    // saturation
+        WBContainer settings;
+        settings.temperature  = 6500.0;
+        settings.dark         = 0.5;
+        settings.black        = 0.0;
+        settings.exposition   = m_customRawSettings.exposureComp;
+        settings.gamma        = 1.0;
+        settings.saturation   = m_customRawSettings.saturation;
+        settings.green        = 1.0;
+        WBFilter wb(m_image, 0L, settings);
+        wb.startFilterDirectly();
+        m_image->putImageData(wb.getTargetImage().bits());       
     }
+    
     if (observer) observer->progressInfo(m_image, 0.92F);
 
     if (m_customRawSettings.lightness != 0.0 ||
         m_customRawSettings.contrast  != 1.0 ||
         m_customRawSettings.gamma     != 1.0)
     {
-        BCGModifier bcg;
-        bcg.setBrightness(m_customRawSettings.lightness);
-        bcg.setContrast(m_customRawSettings.contrast);
-        bcg.setGamma(m_customRawSettings.gamma);
-        bcg.applyBCG(imageData(), imageWidth(), imageHeight(), m_rawDecodingSettings.sixteenBitsImage);
+        BCGContainer settings;
+        settings.brightness = m_customRawSettings.lightness;
+        settings.contrast   = m_customRawSettings.contrast;
+        settings.gamma      = m_customRawSettings.gamma;
+        BCGFilter bcg(m_image, 0L, settings);
+        bcg.startFilterDirectly();
+        m_image->putImageData(bcg.getTargetImage().bits());        
     }
+    
     if (observer) observer->progressInfo(m_image, 0.94F);
 
     if (!m_customRawSettings.curveAdjust.isEmpty())
     {
-        DImg tmp(imageWidth(), imageHeight(), m_rawDecodingSettings.sixteenBitsImage);
-        ImageCurves curves(m_rawDecodingSettings.sixteenBitsImage);
-        curves.setCurvePoints(LuminosityChannel, m_customRawSettings.curveAdjust);
-        curves.curvesCalculateCurve(LuminosityChannel);
-        curves.curvesLutSetup(AlphaChannel);
-        curves.curvesLutProcess(imageData(), tmp.bits(), imageWidth(), imageHeight());
-        memcpy(imageData(), tmp.bits(), tmp.numBytes());
+        CurvesContainer settings;
+        settings.curvesType   = ImageCurves::CURVE_SMOOTH;
+        settings.lumCurveVals = m_customRawSettings.curveAdjust;
+        CurvesFilter curves(m_image, 0L, settings);
+        curves.startFilterDirectly();
+        m_image->putImageData(curves.getTargetImage().bits());
     }
+    
     if (observer) observer->progressInfo(m_image, 0.96F);
 
     if (!m_customRawSettings.levelsAdjust.isEmpty())
@@ -430,6 +360,7 @@ void RAWLoader::postProcess(DImgLoaderObserver *observer)
         levels.levelsLutProcess(imageData(), tmp.bits(), imageWidth(), imageHeight());
         memcpy(imageData(), tmp.bits(), tmp.numBytes());
     }
+    
     if (observer) observer->progressInfo(m_image, 0.98F);
 }
 
