@@ -4,10 +4,9 @@
  * http://www.digikam.org
  *
  * Date        : 2006-01-10
- * Description : a widget to display CIE tongue from
- *               an ICC profile.
+ * Description : a widget to display CIE tongue from an ICC profile.
  *
- * Copyright (C) 2006-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2006-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
  *
  * Any source code are inspired from lprof project and
  * Copyright (C) 1998-2001 Marti Maria
@@ -25,14 +24,13 @@
  *
  * ============================================================ */
 
-#include "cietonguewidget.h"
 #include "cietonguewidget.moc"
 
-// C++ includes.
+// C++ includes
 
 #include <cmath>
 
-// Qt includes.
+// Qt includes
 
 #include <QImage>
 #include <QPainter>
@@ -41,32 +39,33 @@
 #include <QTimer>
 #include <QPaintEvent>
 
-// KDE includes.
+// KDE includes
+
 
-#include <kdebug.h>
 #include <klocale.h>
+#include <kiconloader.h>
 
-// Local includes.
+// Local includes
 
+#include "iccprofile.h"
 #include "lcmsprf.h"
 
 namespace Digikam
 {
 
     /**
-    The  following  table  gives  the  CIE  color  matching  functions
-    \bar{x}(\lambda),  \bar{y}(\lambda),  and   \bar{z}(\lambda),   for
-    wavelengths  \lambda  at 5 nanometer increments from 380 nm through
-    780 nm.  This table is used in conjunction with  Planck's  law  for
-    the  energy spectrum of a black body at a given temperature to plot
-    the black body curve on the CIE chart.
+    The following table gives the CIE  color matching functions
+    \f$\bar{x}(\lambda)\f$, \f$\bar{y}(\lambda)\f$, and
+    \f$\bar{z}(\lambda)\f$, for wavelengths \f$\lambda\f$ at 5 nanometer
+    increments from 380 nm through 780 nm. This table is used in conjunction
+    with Planck's law for the energy spectrum of a black body at a given
+    temperature to plot the black body curve on the CIE chart.
 
-    The following table gives the  spectral  chromaticity  co-ordinates
-    x(\lambda) and y(\lambda) for wavelengths in 5 nanometer increments
-    from 380 nm through  780  nm.   These  co-ordinates  represent  the
+    The following table gives the spectral chromaticity co-ordinates
+    \f$x(\lambda)\f$ and \f$y(\lambda)\f$ for wavelengths in 5 nanometer
+    increments from 380 nm through 780 nm. These coordinates represent the
     position in the CIE x-y space of pure spectral colors of the given
-    wavelength, and  thus  define  the  outline  of  the  CIE  "tongue"
-    diagram.
+    wavelength, and thus define the outline of the CIE "tongue" diagram.
     */
     static const double spectral_chromaticity[81][3] =
     {
@@ -159,37 +158,41 @@ public:
 
     CIETongueWidgetPriv()
     {
-        hMonitorProfile      = 0;
-        hXYZProfile          = 0;
-        hXFORM               = 0;
-
-        Measurement.Patches  = 0;
-        Measurement.Allowed  = 0;
-        blinkTimer           = 0;
-        pos                  = 0;
-
         profileDataAvailable = true;
         loadingImageMode     = false;
         loadingImageSucess   = false;
         needUpdatePixmap     = false;
+        uncalibratedColor    = false;
+        hMonitorProfile      = 0;
+        hXYZProfile          = 0;
+        hXFORM               = 0;
+        Measurement.Patches  = 0;
+        Measurement.Allowed  = 0;
+        progressCount        = 0;
+        progressTimer        = 0;
+        progressPix          = SmallIcon("process-working", 22);
     }
 
     bool             profileDataAvailable;
     bool             loadingImageMode;
     bool             loadingImageSucess;
     bool             needUpdatePixmap;
-
-    double           gridside;
+    bool             uncalibratedColor;
 
     int              xBias;
     int              yBias;
     int              pxcols;
     int              pxrows;
-    int              pos;           // Position of animation during loading/calculation.
+    int              progressCount;           // Position of animation during loading/calculation.
+
+    double           gridside;
 
     QPainter         painter;
+
+    QTimer          *progressTimer;
+
     QPixmap          pixmap;
-    QTimer          *blinkTimer;
+    QPixmap          progressPix;
 
     cmsHPROFILE      hMonitorProfile;
     cmsHPROFILE      hXYZProfile;
@@ -203,7 +206,7 @@ public:
 CIETongueWidget::CIETongueWidget(int w, int h, QWidget *parent, cmsHPROFILE hMonitor)
                : QWidget(parent), d(new CIETongueWidgetPriv)
 {
-    d->blinkTimer = new QTimer(this);
+    d->progressTimer = new QTimer(this);
     setMinimumSize(w, h);
     setAttribute(Qt::WA_DeleteOnClose);
     cmsErrorAction(LCMS_ERROR_SHOW);
@@ -218,8 +221,8 @@ CIETongueWidget::CIETongueWidget(int w, int h, QWidget *parent, cmsHPROFILE hMon
                                         d->hMonitorProfile, TYPE_RGB_8,
                                         INTENT_PERCEPTUAL, 0);
 
-    connect(d->blinkTimer, SIGNAL(timeout()),
-            this, SLOT(slotBlinkTimerDone()));
+    connect(d->progressTimer, SIGNAL(timeout()),
+            this, SLOT(slotProgressTimerDone()));
 }
 
 CIETongueWidget::~CIETongueWidget()
@@ -241,12 +244,12 @@ int CIETongueWidget::grids(double val) const
     return (int) floor(val * d->gridside + 0.5);
 }
 
-bool CIETongueWidget::setProfileData(const QByteArray &profileData)
+bool CIETongueWidget::setProfileData(const QByteArray& profileData)
 {
     if (!profileData.isEmpty())
     {
-        cmsHPROFILE hProfile = cmsOpenProfileFromMem((void*)profileData.data(),
-                                                    (DWORD)profileData.size());
+        LcmsLock lock;
+        cmsHPROFILE hProfile = cmsOpenProfileFromMem((void*)profileData.data(), (DWORD)profileData.size());
 
         if (!hProfile)
         {
@@ -268,8 +271,9 @@ bool CIETongueWidget::setProfileData(const QByteArray &profileData)
     }
 
     d->loadingImageMode = false;
+    d->uncalibratedColor = false;
 
-    d->blinkTimer->stop();
+    d->progressTimer->stop();
     d->needUpdatePixmap = true;
     update();
     return (d->profileDataAvailable);
@@ -279,7 +283,8 @@ bool CIETongueWidget::setProfileFromFile(const KUrl& file)
 {
     if (!file.isEmpty() && file.isValid())
     {
-        cmsHPROFILE hProfile = cmsOpenProfileFromFile(QFile::encodeName(file.path()), "r");
+        LcmsLock lock;
+        cmsHPROFILE hProfile = cmsOpenProfileFromFile(QFile::encodeName(file.toLocalFile()), "r");
 
         if (!hProfile)
         {
@@ -300,7 +305,10 @@ bool CIETongueWidget::setProfileFromFile(const KUrl& file)
         d->loadingImageSucess   = false;
     }
 
-    d->blinkTimer->stop();
+    d->loadingImageMode = false;
+    d->uncalibratedColor = false;
+
+    d->progressTimer->stop();
     d->needUpdatePixmap = true;
     update();
     return (d->profileDataAvailable);
@@ -319,7 +327,7 @@ void CIETongueWidget::setProfile(cmsHPROFILE hProfile)
 
     ZeroMemory(&(d->Primaries), sizeof(cmsCIExyYTRIPLE));
 
-    if (cmsIsTag(hProfile, icSigRedColorantTag) &&
+    if (cmsIsTag(hProfile, icSigRedColorantTag)   &&
         cmsIsTag(hProfile, icSigGreenColorantTag) &&
         cmsIsTag(hProfile, icSigBlueColorantTag))
     {
@@ -383,9 +391,9 @@ void CIETongueWidget::biasedLine(int x1, int y1, int x2, int y2)
     d->painter.drawLine(x1 + d->xBias, y1, x2 + d->xBias, y2);
 }
 
-void CIETongueWidget::biasedText(int x, int y, QString Txt)
+void CIETongueWidget::biasedText(int x, int y, const QString& txt)
 {
-    d->painter.drawText(QPoint(d->xBias + x, y), Txt);
+    d->painter.drawText(QPoint(d->xBias + x, y), txt);
 }
 
 QRgb CIETongueWidget::colorByCoord(double x, double y)
@@ -449,17 +457,17 @@ void CIETongueWidget::fillTongue()
 
     int x;
 
-    for (int y = 0; y < d->pxrows; y++)
+    for (int y = 0; y < d->pxrows; ++y)
     {
         int xe = 0;
 
         // Find horizontal extents of tongue on this line.
 
-        for (x = 0; x < d->pxcols; x++)
+        for (x = 0; x < d->pxcols; ++x)
         {
             if (QColor(Img.pixel(x + d->xBias, y)) != QColor(Qt::black))
             {
-                for (xe = d->pxcols - 1; xe >= x; xe--)
+                for (xe = d->pxcols - 1; xe >= x; --xe)
                 {
                     if (QColor(Img.pixel(xe + d->xBias, y)) != QColor(Qt::black))
                     {
@@ -473,7 +481,7 @@ void CIETongueWidget::fillTongue()
 
         if (x < d->pxcols)
         {
-            for ( ; x <= xe; x++)
+            for ( ; x <= xe; ++x)
             {
                 QRgb Color = colorByCoord(x, y);
                 Img.setPixel(x + d->xBias, y, Color);
@@ -492,7 +500,7 @@ void CIETongueWidget::drawTongueAxis()
 
     d->painter.setPen(qRgb(255, 255, 255));
 
-    biasedLine(0, 0,          0,          d->pxrows - 1);
+    biasedLine(0, 0,           0,           d->pxrows - 1);
     biasedLine(0, d->pxrows-1, d->pxcols-1, d->pxrows - 1);
 
     for (int y = 1; y <= 9; y += 1)
@@ -584,7 +592,7 @@ void CIETongueWidget::drawSmallElipse(LPcmsCIExyY xyY, BYTE r, BYTE g, BYTE b, i
 
 void CIETongueWidget::drawPatches()
 {
-    for (int i=0; i < d->Measurement.nPatches; i++)
+    for (int i=0; i < d->Measurement.nPatches; ++i)
     {
         LPPATCH p = d->Measurement.Patches + i;
 
@@ -688,22 +696,32 @@ void CIETongueWidget::drawWhitePoint()
 
 void CIETongueWidget::loadingStarted()
 {
-    d->pos                = 0;
+    d->progressCount      = 0;
     d->loadingImageMode   = true;
     d->loadingImageSucess = false;
     update();
-    d->blinkTimer->start(200);
+    d->progressTimer->start(200);
 }
 
 void CIETongueWidget::loadingFailed()
 {
-    d->blinkTimer->stop();
-    d->pos                = 0;
+    d->progressTimer->stop();
+    d->progressCount      = 0;
     d->loadingImageMode   = false;
     d->loadingImageSucess = false;
     update();
 }
 
+void CIETongueWidget::uncalibratedColor()
+{
+    d->progressTimer->stop();
+    d->progressCount      = 0;
+    d->loadingImageMode   = false;
+    d->loadingImageSucess = false;
+    d->uncalibratedColor  = true;
+    update();
+}
+
 void CIETongueWidget::updatePixmap()
 {
     d->needUpdatePixmap = false;
@@ -775,28 +793,14 @@ void CIETongueWidget::paintEvent(QPaintEvent*)
     {
         // In first, we draw an animation.
 
-        int asize = 24;
-        QPixmap anim(asize, asize);
-        QPainter p2;
-        p2.begin(&anim);
-        p2.fillRect(0, 0, asize, asize, palette().color(QPalette::Active, QPalette::Background));
-        p2.translate(asize/2, asize/2);
-
-        d->pos = (d->pos + 10) % 360;
-        p2.setPen(QPen(palette().color(QPalette::Active, QPalette::Text)));
-
-        p2.rotate(d->pos);
-        for ( int i=0 ; i<12 ; i++ )
-        {
-            p2.drawLine(asize/2-5, 0, asize/2-2, 0);
-            p2.rotate(30);
-        }
-        p2.end();
+        QPixmap anim(d->progressPix.copy(0, d->progressCount*22, 22, 22));
+        d->progressCount++;
+        if (d->progressCount == 8) d->progressCount = 0;
 
         // ... and we render busy text.
 
         p.fillRect(0, 0, width(), height(), palette().color(QPalette::Active, QPalette::Background));
-        p.drawPixmap(width()/2 - asize /2, asize, anim);
+        p.drawPixmap(width()/2 - anim.width() /2, anim.height(), anim);
         QPen pen(palette().color(QPalette::Active, QPalette::Text));
         pen.setStyle(Qt::SolidLine);
         pen.setWidth(1);
@@ -809,7 +813,7 @@ void CIETongueWidget::paintEvent(QPaintEvent*)
         return;
     }
 
-    // No profile data to show.
+    // No profile data to show, or RAW file
 
     if (!d->profileDataAvailable || (!d->loadingImageMode && !d->loadingImageSucess))
     {
@@ -820,9 +824,18 @@ void CIETongueWidget::paintEvent(QPaintEvent*)
 
         p.setPen(pen);
         p.drawRect(0, 0, width(), height());
-        p.setPen(Qt::red);
-        p.drawText(0, 0, width(), height(), Qt::AlignCenter,
-                   i18n("No profile available..."));
+
+        if (d->uncalibratedColor)
+        {
+            p.drawText(0, 0, width(), height(), Qt::AlignCenter,
+                    i18n("Uncalibrated color space"));
+        }
+        else
+        {
+            p.setPen(Qt::red);
+            p.drawText(0, 0, width(), height(), Qt::AlignCenter,
+                    i18n("No profile available..."));
+        }
 
         return;
     }
@@ -835,10 +848,16 @@ void CIETongueWidget::paintEvent(QPaintEvent*)
     p.drawPixmap(0, 0, d->pixmap);
 }
 
-void CIETongueWidget::slotBlinkTimerDone()
+void CIETongueWidget::resizeEvent(QResizeEvent * event)
+{
+    Q_UNUSED(event);
+    d->needUpdatePixmap = true;
+}
+
+void CIETongueWidget::slotProgressTimerDone()
 {
     update();
-    d->blinkTimer->start( 200 );
+    d->progressTimer->start(200);
 }
 
 }  // namespace Digikam
