@@ -23,22 +23,21 @@
  *
  * ============================================================ */
 
-#include "cameracontroller.h"
 #include "cameracontroller.moc"
 
-// C ANSI includes.
+// C ANSI includes
 
 extern "C"
 {
 #include <unistd.h>
 }
 
-// C++ includes.
+// C++ includes
 
 #include <typeinfo>
 #include <cstdio>
 
-// Qt includes.
+// Qt includes
 
 #include <QMutex>
 #include <QWaitCondition>
@@ -47,19 +46,21 @@ extern "C"
 #include <QFile>
 #include <QRegExp>
 #include <QFileInfo>
+#include <QPointer>
 
 // KDE includes
 
 #include <kiconloader.h>
-#include <kdebug.h>
 #include <kio/renamedialog.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kstandarddirs.h>
 #include <kurl.h>
+#include <kdebug.h>
 
-// Local includes.
+// Local includes
 
+#include "templatemanager.h"
 #include "thumbnailsize.h"
 #include "imagewindow.h"
 #include "gpcamera.h"
@@ -86,7 +87,7 @@ public:
         gp_upload,
         gp_delete,
         gp_lock,
-        gp_thumbnail,
+        gp_thumbnails,
         gp_exif,
         gp_open,
         gp_freeSpace,
@@ -123,11 +124,11 @@ public:
 
     int                     downloadTotal;
 
-    QWidget                *parent;
+    QWidget*                parent;
 
-    QTimer                 *timer;
+    QTimer*                 timer;
 
-    DKCamera               *camera;
+    DKCamera*               camera;
 
     QMutex                  mutex;
     QWaitCondition          condVar;
@@ -143,20 +144,20 @@ CameraController::CameraController(QWidget* parent,
     d->parent = parent;
 
     // URL parsing (c) Stephan Kulow
-    if (path.startsWith("camera:/"))
+    if (path.startsWith(QLatin1String("camera:/")))
     {
         KUrl url(path);
-        kDebug(50003) << "path " << path << " " << url <<  " " << url.host() << endl;
+        kDebug() << "path " << path << " " << url <<  " " << url.host();
         QString xport = url.host();
-        if (xport.startsWith("usb:"))
+        if (xport.startsWith(QLatin1String("usb:")))
         {
-            kDebug(50003) << "xport " << xport << endl;
+            kDebug() << "xport " << xport;
             QRegExp x = QRegExp("(usb:[0-9,]*)");
 
             if (x.indexIn(xport) != -1)
             {
                 QString usbport = x.cap(1);
-                kDebug(50003) << "USB " << xport << " " << usbport << endl;
+                kDebug() << "USB " << xport << " " << usbport;
                 // if ((xport == usbport) || ((count == 1) && (xport == "usb:"))) {
                 //   model = xmodel;
                 d->camera = new GPCamera(title, url.user(), "usb:", "/");
@@ -289,13 +290,13 @@ QPixmap CameraController::mimeTypeThumbnail(const QString& itemName)
     QFileInfo fi(itemName);
     QString mime = d->camera->mimeType(fi.suffix().toLower());
 
-    if (mime.startsWith("image/x-raw"))
+    if (mime.startsWith(QLatin1String("image/x-raw")))
         return DesktopIcon("kdcraw");
-    else if (mime.startsWith("image/"))
+    else if (mime.startsWith(QLatin1String("image/")))
         return DesktopIcon("image-x-generic");
-    else if (mime.startsWith("video/"))
+    else if (mime.startsWith(QLatin1String("video/")))
         return DesktopIcon("video-x-generic");
-    else if (mime.startsWith("audio/"))
+    else if (mime.startsWith(QLatin1String("audio/")))
         return DesktopIcon("audio-x-generic");
 
     return DesktopIcon("unknown");
@@ -342,22 +343,22 @@ void CameraController::executeCommand(CameraCommand *cmd)
     {
         case(CameraCommand::gp_connect):
         {
-            sendInfo(i18n("Connecting to camera..."));
+            sendLogMsg(i18n("Connecting to camera..."));
 
             bool result = d->camera->doConnect();
 
             emit signalConnected(result);
 
             if (result)
-                sendInfo(i18n("Connection established"));
+                sendLogMsg(i18n("Connection established."));
             else
-                sendInfo(i18n("Connection failed"));
+                sendLogMsg(i18n("Connection failed."));
 
             break;
         }
         case(CameraCommand::gp_cameraInformation):
         {
-            sendInfo(i18n("Getting camera information..."));
+            sendLogMsg(i18n("Getting camera information..."));
 
             QString summary, manual, about;
 
@@ -370,7 +371,7 @@ void CameraController::executeCommand(CameraCommand *cmd)
         }
         case(CameraCommand::gp_freeSpace):
         {
-            sendInfo(i18n("Getting camera freespace available..."));
+            sendLogMsg(i18n("Getting available free space on camera..."));
             unsigned long kBSize  = 0;
             unsigned long kBAvail = 0;
             d->camera->getFreeSpace(kBSize, kBAvail);
@@ -379,7 +380,7 @@ void CameraController::executeCommand(CameraCommand *cmd)
         }
         case(CameraCommand::gp_preview):
         {
-            sendInfo(i18n("Getting preview..."));
+            sendLogMsg(i18n("Getting preview..."));
             QImage preview;
             d->camera->getPreview(preview);
             emit signalPreview(preview);
@@ -387,7 +388,7 @@ void CameraController::executeCommand(CameraCommand *cmd)
         }
         case(CameraCommand::gp_capture):
         {
-            sendInfo(i18n("Capture image..."));
+            sendLogMsg(i18n("Capture image..."));
             GPItemInfo itemInfo;
             d->camera->capture(itemInfo);
             emit signalUploaded(itemInfo);
@@ -395,14 +396,14 @@ void CameraController::executeCommand(CameraCommand *cmd)
         }
         case(CameraCommand::gp_listfolders):
         {
-            sendInfo(i18n("Listing folders..."));
+            sendLogMsg(i18n("Listing folders..."));
 
             QStringList folderList;
             folderList.append(d->camera->path());
             d->camera->getAllFolders(d->camera->path(), folderList);
 
             emit signalFolderList(folderList);
-            sendInfo(i18n("The folders have been listed."));
+            sendLogMsg(i18n("The folders have been listed."));
 
             break;
         }
@@ -410,12 +411,12 @@ void CameraController::executeCommand(CameraCommand *cmd)
         {
             QString folder = cmd->map["folder"].toString();
 
-            sendInfo(i18n("Listing files in %1...", folder));
+            sendLogMsg(i18n("Listing files in %1...", folder));
 
             GPItemInfoList itemsList;
             if (!d->camera->getItemsInfoList(folder, itemsList, true))
             {
-                sendError(i18n("Failed to list files in %1", folder));
+                sendLogMsg(i18n("Failed to list files in %1.", folder), DHistoryView::ErrorEntry);
             }
 
             if (!itemsList.isEmpty())
@@ -423,27 +424,36 @@ void CameraController::executeCommand(CameraCommand *cmd)
                 emit signalFileList(itemsList);
             }
 
-            sendInfo(i18n("The files in %1 have been listed.", folder));
+            sendLogMsg(i18n("The files in %1 have been listed.", folder));
 
             break;
         }
-        case(CameraCommand::gp_thumbnail):
+        case(CameraCommand::gp_thumbnails):
         {
-            QString folder = cmd->map["folder"].toString();
-            QString file   = cmd->map["file"].toString();
+            QList<QVariant> list = cmd->map["list"].toList();
 
-            sendInfo(i18n("Getting thumbnails..."));
-
-            QImage thumbnail;
-
-            if (d->camera->getThumbnail(folder, file, thumbnail))
+            for (QList<QVariant>::const_iterator it = list.constBegin(); it != list.constEnd(); ++it)
             {
-                thumbnail = thumbnail.scaled(ThumbnailSize::Huge, ThumbnailSize::Huge,
-                                             Qt::KeepAspectRatio);
+                if (d->canceled)
+                {
+                    break;
+                }
+                QString folder = (*it).toStringList()[0];
+                QString file   = (*it).toStringList()[1];
+
+                sendLogMsg(i18n("Getting thumbnails for %1...", file), DHistoryView::StartingEntry, folder, file);
+                QImage thumbnail;
+
+                if (d->camera->getThumbnail(folder, file, thumbnail))
+                {
+                    thumbnail = thumbnail.scaled(ThumbnailSize::Huge, ThumbnailSize::Huge, Qt::KeepAspectRatio);
+                    emit signalThumbnail(folder, file, thumbnail);
+                }
+                else
+                {
+                    emit signalThumbnailFailed(folder, file);
+                }
             }
-
-            emit signalThumbnail(folder, file, thumbnail);
-
             break;
         }
         case(CameraCommand::gp_exif):
@@ -457,7 +467,7 @@ void CameraController::executeCommand(CameraCommand *cmd)
             }
             else
             {
-                sendInfo(i18n("Getting EXIF information for %1...", file));
+                sendLogMsg(i18n("Getting EXIF information for %1...", file), DHistoryView::StartingEntry, folder, file);
 
                 char* edata = 0;
                 int   esize = 0;
@@ -477,67 +487,77 @@ void CameraController::executeCommand(CameraCommand *cmd)
         }
         case(CameraCommand::gp_download):
         {
-            QString   folder            = cmd->map["folder"].toString();
-            QString   file              = cmd->map["file"].toString();
-            QString   dest              = cmd->map["dest"].toString();
-            bool      autoRotate        = cmd->map["autoRotate"].toBool();
-            bool      fixDateTime       = cmd->map["fixDateTime"].toBool();
-            QDateTime newDateTime       = cmd->map["newDateTime"].toDateTime();
-            bool      setPhotographerId = cmd->map["setPhotographerId"].toBool();
-            QString   author            = cmd->map["author"].toString();
-            QString   authorTitle       = cmd->map["authorTitle"].toString();
-            bool      setCredits        = cmd->map["setCredits"].toBool();
-            QString   credit            = cmd->map["credit"].toString();
-            QString   source            = cmd->map["source"].toString();
-            QString   copyright         = cmd->map["copyright"].toString();
-            bool      convertJpeg       = cmd->map["convertJpeg"].toBool();
-            QString   losslessFormat    = cmd->map["losslessFormat"].toString();
-            sendInfo(i18n("Downloading file %1...", file));
+            QString   folder         = cmd->map["folder"].toString();
+            QString   file           = cmd->map["file"].toString();
+            QString   dest           = cmd->map["dest"].toString();
+            bool      autoRotate     = cmd->map["autoRotate"].toBool();
+            bool      fixDateTime    = cmd->map["fixDateTime"].toBool();
+            QDateTime newDateTime    = cmd->map["newDateTime"].toDateTime();
+            QString   templateTitle  = cmd->map["template"].toString();
+            bool      convertJpeg    = cmd->map["convertJpeg"].toBool();
+            QString   losslessFormat = cmd->map["losslessFormat"].toString();
+            sendLogMsg(i18n("Downloading file %1...", file), DHistoryView::StartingEntry, folder, file);
 
             // download to a temp file
 
             emit signalDownloaded(folder, file, GPItemInfo::DownloadStarted);
 
             KUrl tempURL(dest);
-            tempURL      = tempURL.upUrl();
+            tempURL = tempURL.upUrl();
             tempURL.addPath(QString(".digikam-camera-tmp1-%1").arg(getpid()).append(file));
-            kDebug(50003) << "Downloading: " << file << " using (" << tempURL << ")" << endl;
-            QString temp = tempURL.path();
+            kDebug() << "Downloading: " << file << " using (" << tempURL << ")";
+            QString temp = tempURL.toLocalFile();
 
-            bool result  = d->camera->downloadItem(folder, file, tempURL.path());
+            bool result  = d->camera->downloadItem(folder, file, tempURL.toLocalFile());
 
             if (!result)
             {
-                unlink(QFile::encodeName(tempURL.path()));
+                unlink(QFile::encodeName(tempURL.toLocalFile()));
                 emit signalDownloaded(folder, file, GPItemInfo::DownloadFailed);
+                sendLogMsg(i18n("Failed to download %1...", file), DHistoryView::ErrorEntry, folder, file);
                 break;
             }
-            else if (isJpegImage(tempURL.path()))
+            else if (isJpegImage(tempURL.toLocalFile()))
             {
                 // Possible modification operations. Only apply it to JPEG for the moment.
 
                 if (autoRotate)
                 {
-                    kDebug(50003) << "Exif autorotate: " << file << " using (" << tempURL << ")" << endl;
-                    sendInfo(i18n("EXIF rotating file %1...", file));
-                    exifRotate(tempURL.path(), file);
+                    kDebug() << "Exif autorotate: " << file << " using (" << tempURL << ")";
+                    sendLogMsg(i18n("EXIF rotating file %1...", file), DHistoryView::StartingEntry, folder, file);
+                    exifTransform(tempURL.toLocalFile(), file);
                 }
 
-                if (fixDateTime || setPhotographerId || setCredits)
+                if (!templateTitle.isNull() || fixDateTime)
                 {
-                    kDebug(50003) << "Set metadata from: " << file << " using (" << tempURL << ")" << endl;
-                    sendInfo(i18n("Setting Metadata tags to file %1...", file));
-                    DMetadata metadata(tempURL.path());
+                    kDebug() << "Set metadata from: " << file << " using (" << tempURL << ")";
+                    DMetadata metadata(tempURL.toLocalFile());
 
                     if (fixDateTime)
+                    {
+                        sendLogMsg(i18n("Fix Internal date to file %1...", file), DHistoryView::StartingEntry, folder, file);
                         metadata.setImageDateTime(newDateTime, true);
+                    }
 
-                    if (setPhotographerId)
-                        metadata.setImagePhotographerId(author, authorTitle);
-
-                    if (setCredits)
-                        metadata.setImageCredits(credit, source, copyright);
-
+                    TemplateManager* tm = TemplateManager::defaultManager();
+                    if (tm && !templateTitle.isEmpty())
+                    {
+                        kDebug() << "Metadata template title : " << templateTitle;
+                        if (templateTitle == Template::removeTemplateTitle())
+                        {
+                            metadata.removeMetadataTemplate();
+                        }
+                        else if (templateTitle.isEmpty())
+                        {
+                            // Nothing to do.
+                        }
+                        else
+                        {
+                            sendLogMsg(i18n("Apply Metadata template to file %1...", file), DHistoryView::StartingEntry, folder, file);
+                            metadata.removeMetadataTemplate();
+                            metadata.setMetadataTemplate(tm->findByTitle(templateTitle));
+                        }
+                    }
                     metadata.applyChanges();
                 }
 
@@ -546,25 +566,25 @@ void CameraController::executeCommand(CameraCommand *cmd)
 
                 if (convertJpeg)
                 {
-                    kDebug(50003) << "Convert to LossLess: " << file << " using (" << tempURL << ")" << endl;
-                    sendInfo(i18n("Converting %1 to lossless file format...", file));
+                    kDebug() << "Convert to LossLess: " << file << " using (" << tempURL << ")";
+                    sendLogMsg(i18n("Converting %1 to lossless file format...", file), DHistoryView::StartingEntry, folder, file);
 
                     KUrl tempURL2(dest);
                     tempURL2 = tempURL2.upUrl();
                     tempURL2.addPath(QString(".digikam-camera-tmp2-%1").arg(getpid()).append(file));
-                    temp     = tempURL2.path();
+                    temp     = tempURL2.toLocalFile();
 
-                    if (!jpegConvert(tempURL.path(), tempURL2.path(), file, losslessFormat))
+                    if (!jpegConvert(tempURL.toLocalFile(), tempURL2.toLocalFile(), file, losslessFormat))
                     {
                         // convert failed. delete the temp file
-                        unlink(QFile::encodeName(tempURL.path()));
-                        unlink(QFile::encodeName(tempURL2.path()));
+                        unlink(QFile::encodeName(tempURL.toLocalFile()));
+                        unlink(QFile::encodeName(tempURL2.toLocalFile()));
                         result = false;
                     }
                     else
                     {
                         // Else remove only the first temp file.
-                        unlink(QFile::encodeName(tempURL.path()));
+                        unlink(QFile::encodeName(tempURL.toLocalFile()));
                     }
                 }
             }
@@ -580,7 +600,7 @@ void CameraController::executeCommand(CameraCommand *cmd)
             QString file   = cmd->map["file"].toString();
             QString dest   = cmd->map["dest"].toString();
 
-            sendInfo(i18n("Retrieving file %1 from camera...", file));
+            sendLogMsg(i18n("Retrieving file %1 from camera...", file), DHistoryView::StartingEntry, folder, file);
 
             bool result = d->camera->downloadItem(folder, file, dest);
 
@@ -590,7 +610,7 @@ void CameraController::executeCommand(CameraCommand *cmd)
             }
             else
             {
-                sendError(i18n("Failed to retrieve file %1 from camera", file));
+                sendLogMsg(i18n("Failed to retrieve file %1 from camera.", file), DHistoryView::ErrorEntry, folder, file);
             }
             break;
         }
@@ -605,7 +625,7 @@ void CameraController::executeCommand(CameraCommand *cmd)
             // The source file path to download in camera.
             QString src    = cmd->map["srcFilePath"].toString();
 
-            sendInfo(i18n("Uploading file %1 to camera...", file));
+            sendLogMsg(i18n("Uploading file %1 to camera...", file), DHistoryView::StartingEntry, folder, file);
 
             GPItemInfo itemsInfo;
 
@@ -626,7 +646,7 @@ void CameraController::executeCommand(CameraCommand *cmd)
             QString folder = cmd->map["folder"].toString();
             QString file   = cmd->map["file"].toString();
 
-            sendInfo(i18n("Deleting file %1...", file));
+            sendLogMsg(i18n("Deleting file %1...", file), DHistoryView::StartingEntry, folder, file);
 
             bool result = d->camera->deleteItem(folder, file);
 
@@ -646,7 +666,7 @@ void CameraController::executeCommand(CameraCommand *cmd)
             QString file   = cmd->map["file"].toString();
             bool    lock   = cmd->map["lock"].toBool();
 
-            sendInfo(i18n("Toggle lock file %1...", file));
+            sendLogMsg(i18n("Toggle lock file %1...", file), DHistoryView::StartingEntry, folder, file);
 
             bool result = d->camera->setLockItem(folder, file, lock);
 
@@ -662,7 +682,7 @@ void CameraController::executeCommand(CameraCommand *cmd)
         }
         default:
         {
-            kWarning(50003) << " unknown action specified" << endl;
+            kWarning() << " unknown action specified";
         }
     }
 }
@@ -672,19 +692,15 @@ void CameraController::sendBusy(bool val)
     emit signalBusy(val);
 }
 
-void CameraController::sendError(const QString& msg)
-{
-    emit signalErrorMsg(msg);
-}
-
-void CameraController::sendInfo(const QString& msg)
+void CameraController::sendLogMsg(const QString& msg, DHistoryView::EntryType type,
+                                  const QString& folder, const QString& file)
 {
     if (!d->canceled)
-        emit signalInfoMsg(msg);
+        emit signalLogMsg(msg, type, folder, file);
 }
 
-void CameraController::slotCheckRename(const QString &folder, const QString &file,
-                                       const QString &destination, const QString &temp)
+void CameraController::slotCheckRename(const QString& folder, const QString& file,
+                                       const QString& destination, const QString& temp)
 {
     // this is the direct continuation of executeCommand, case CameraCommand::gp_download
 
@@ -695,9 +711,9 @@ void CameraController::slotCheckRename(const QString &folder, const QString &fil
 
     // Check if dest file already exist, unless we overwrite anyway
 
+    QFileInfo info(dest);
     if (!d->overwriteAll)
     {
-        QFileInfo info(dest);
 
         while (info.exists())
         {
@@ -707,14 +723,18 @@ void CameraController::slotCheckRename(const QString &folder, const QString &fil
                 break;
             }
 
-            KIO::RenameDialog dlg(d->parent, i18n("Rename File"),
-                                  folder + QString("/") + file, dest,
-                                  KIO::RenameDialog_Mode(KIO::M_MULTI | KIO::M_OVERWRITE | KIO::M_SKIP));
+            QPointer<KIO::RenameDialog> dlg = new KIO::RenameDialog(d->parent, i18n("Rename File"),
+                                                                    folder + QString("/") + file, dest,
+                                                                    KIO::RenameDialog_Mode(KIO::M_MULTI |
+                                                                    KIO::M_OVERWRITE                    |
+                                                                    KIO::M_SKIP));
 
-            int result = dlg.exec();
-            dest       = dlg.newDestUrl().path();
+            int result = dlg->exec();
+            dest       = dlg->newDestUrl().toLocalFile();
             info       = QFileInfo(dest);
 
+            delete dlg;
+
             switch (result)
             {
                 case KIO::R_CANCEL:
@@ -763,7 +783,7 @@ void CameraController::slotCheckRename(const QString &folder, const QString &fil
     else if (skip)
     {
         unlink(QFile::encodeName(temp));
-        emit signalInfoMsg(i18n("Skipped file %1", file));
+        sendLogMsg(i18n("Skipped file %1", file), DHistoryView::WarningEntry, folder, file);
         emit signalSkipped(folder, file);
         return;
     }
@@ -774,17 +794,20 @@ void CameraController::slotCheckRename(const QString &folder, const QString &fil
         // rename failed. delete the temp file
         unlink(QFile::encodeName(temp));
         emit signalDownloaded(folder, file, GPItemInfo::DownloadFailed);
+        sendLogMsg(i18n("Failed to download %1...", file), DHistoryView::ErrorEntry,  folder, file);
     }
     else
     {
         emit signalDownloaded(folder, file, GPItemInfo::DownloadedYes);
+        emit signalDownloadComplete(folder, file, info.path(), info.fileName());
+        sendLogMsg(i18n("Download successfully %1...", file), DHistoryView::StartingEntry, folder, file);
     }
 }
 
-void CameraController::slotDownloadFailed(const QString &folder, const QString &file)
+void CameraController::slotDownloadFailed(const QString& folder, const QString& file)
 {
-    Q_UNUSED(folder);
     QString msg = i18n("Failed to download file \"%1\".", file);
+    sendLogMsg(i18n("Failed to download %1...", file), DHistoryView::ErrorEntry, folder, file);
 
     if (!d->canceled)
     {
@@ -802,12 +825,13 @@ void CameraController::slotDownloadFailed(const QString &folder, const QString &
     }
 }
 
-void CameraController::slotUploadFailed(const QString &folder, const QString &file, const QString &src)
+void CameraController::slotUploadFailed(const QString& folder, const QString& file, const QString& src)
 {
     Q_UNUSED(folder);
     Q_UNUSED(src);
 
-    QString msg = i18n("Failed to upload file \"%1\".",file);
+    QString msg = i18n("Failed to upload file \"%1\".", file);
+    sendLogMsg(i18n("Failed to upload %1...", file), DHistoryView::ErrorEntry);
 
     if (!d->canceled)
     {
@@ -825,9 +849,10 @@ void CameraController::slotUploadFailed(const QString &folder, const QString &fi
     }
 }
 
-void CameraController::slotDeleteFailed(const QString &folder, const QString &file)
+void CameraController::slotDeleteFailed(const QString& folder, const QString& file)
 {
     emit signalDeleted(folder, file, false);
+    sendLogMsg(i18n("Failed to delete %1...", file), DHistoryView::ErrorEntry, folder, file);
 
     QString msg = i18n("Failed to delete file \"%1\".",file);
 
@@ -847,9 +872,10 @@ void CameraController::slotDeleteFailed(const QString &folder, const QString &fi
     }
 }
 
-void CameraController::slotLockFailed(const QString &folder, const QString &file)
+void CameraController::slotLockFailed(const QString& folder, const QString& file)
 {
     emit signalLocked(folder, file, false);
+    sendLogMsg(i18n("Failed to lock %1...", file), DHistoryView::ErrorEntry, folder, file);
 
     QString msg = i18n("Failed to toggle lock file \"%1\".",file);
 
@@ -869,7 +895,7 @@ void CameraController::slotLockFailed(const QString &folder, const QString &file
     }
 }
 
-void CameraController::slotOpen(const QString &folder, const QString &file, const QString &dest)
+void CameraController::slotOpen(const QString& folder, const QString& file, const QString& dest)
 {
     Q_UNUSED(folder);
     Q_UNUSED(file);
@@ -928,12 +954,18 @@ void CameraController::listFiles(const QString& folder)
 }
 
 void CameraController::getThumbnail(const QString& folder, const QString& file)
+{
+    QList<QVariant> list;
+    list.append(QStringList() << folder << file);
+    getThumbnails(list);
+}
+
+void CameraController::getThumbnails(const QList<QVariant>& list)
 {
     d->canceled        = false;
     CameraCommand *cmd = new CameraCommand;
-    cmd->action        = CameraCommand::gp_thumbnail;
-    cmd->map.insert("folder", QVariant(folder));
-    cmd->map.insert("file",   QVariant(file));
+    cmd->action        = CameraCommand::gp_thumbnails;
+    cmd->map.insert("list", QVariant(list));
     addCommand(cmd);
 }
 
@@ -988,8 +1020,8 @@ void CameraController::upload(const QFileInfo& srcFileInfo, const QString& destF
     cmd->map.insert("destFile",    QVariant(destFile));
     cmd->map.insert("destFolder",  QVariant(destFolder));
     addCommand(cmd);
-    kDebug(50003) << "Uploading '" << srcFileInfo.filePath() << "' into camera : '" << destFolder <<
-                 "' (" << destFile << ")" << endl;
+    kDebug() << "Uploading '" << srcFileInfo.filePath() << "' into camera : '" << destFolder
+             << "' (" << destFile << ")";
 }
 
 void CameraController::downloadPrep()
@@ -1010,13 +1042,7 @@ void CameraController::download(const DownloadSettingsContainer& downloadSetting
     cmd->map.insert("autoRotate",        QVariant(downloadSettings.autoRotate));
     cmd->map.insert("fixDateTime",       QVariant(downloadSettings.fixDateTime));
     cmd->map.insert("newDateTime",       QVariant(downloadSettings.newDateTime));
-    cmd->map.insert("setPhotographerId", QVariant(downloadSettings.setPhotographerId));
-    cmd->map.insert("author",            QVariant(downloadSettings.author));
-    cmd->map.insert("authorTitle",       QVariant(downloadSettings.authorTitle));
-    cmd->map.insert("setCredits",        QVariant(downloadSettings.setCredits));
-    cmd->map.insert("credit",            QVariant(downloadSettings.credit));
-    cmd->map.insert("source",            QVariant(downloadSettings.source));
-    cmd->map.insert("copyright",         QVariant(downloadSettings.copyright));
+    cmd->map.insert("template",          QVariant(downloadSettings.templateTitle));
     cmd->map.insert("convertJpeg",       QVariant(downloadSettings.convertJpeg));
     cmd->map.insert("losslessFormat",    QVariant(downloadSettings.losslessFormat));
     addCommand(cmd);
