@@ -7,7 +7,7 @@
  * Description : USB Mass Storage camera interface
  *
  * Copyright (C) 2004-2005 by Renchi Raju <renchi@pooh.tam.uiuc.edu>
- * Copyright (C) 2005-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2005-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
  *
  * This program is free software; you can redistribute it
  * and/or modify it under the terms of the GNU General
@@ -24,7 +24,7 @@
 
 #include "umscamera.h"
 
-// C ANSI includes.
+// C ANSI includes
 
 extern "C"
 {
@@ -34,37 +34,36 @@ extern "C"
 #include <utime.h>
 }
 
-// Qt includes.
+// Qt includes
 
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
 #include <QMatrix>
 #include <QStringList>
+#include <QTextDocument>
 
-// KDE includes.
+// KDE includes
 
-#include <kcodecs.h>
 #include <kdebug.h>
+#include <kcodecs.h>
 #include <kio/global.h>
-#include <kio/thumbcreator.h>
-#include <klibloader.h>
 #include <klocale.h>
 #include <kmimetype.h>
-#include <ktrader.h>
 #include <solid/device.h>
 #include <solid/storageaccess.h>
 #include <solid/storagedrive.h>
 #include <solid/storagevolume.h>
 
-// LibKDcraw includes.
+// LibKDcraw includes
 
 #include <libkdcraw/kdcraw.h>
 
-// Local includes.
+// Local includes
 
 #include "dimg.h"
 #include "dmetadata.h"
+#include "imagescanner.h"
 
 namespace Digikam
 {
@@ -85,10 +84,10 @@ QByteArray UMSCamera::cameraMD5ID()
 {
     QString camData;
     // Camera media UUID is used (if available) to improve fingerprint value
-    // registration to database. We want to be sure that MD5 sum is really uniq.
+    // registration to database. We want to be sure that MD5 sum is really unique.
     // We don't use camera information here. media UUID is enough because it came be
-    // mouted by a card reader or a camera. In this case, "already donwloaded" flag will
-    // be undependant of the device used to mount memory card.
+    // mounted by a card reader or a camera. In this case, "already downloaded" flag will
+    // be independent of the device used to mount memory card.
     camData.append(uuid());
     KMD5 md5(camData.toUtf8());
     return md5.hexDigest();
@@ -196,11 +195,8 @@ bool UMSCamera::getItemsInfoList(const QString& folder, GPItemInfoList& infoList
                 pInfo = meta.getPhotographInformation();
             }
 
-            if (dt.isNull())
-            {
-                // If date is not found in metadata, use file time stamp
-                dt = fi->created();
-            }
+            if (dt.isNull()) // fall back to file system info
+                ImageScanner::creationDateFromFilesystem(*fi);
 
             info.name             = fi->fileName();
             info.folder           = !folder.endsWith('/') ? folder + QString('/') : folder;
@@ -209,7 +205,7 @@ bool UMSCamera::getItemsInfoList(const QString& folder, GPItemInfoList& infoList
             info.size             = fi->size();
             info.width            = getImageDimensions ? dims.width()  : -1;
             info.height           = getImageDimensions ? dims.height() : -1;
-            info.downloaded       = GPItemInfo::DownloadUnknow;
+            info.downloaded       = GPItemInfo::DownloadUnknown;
             info.readPermissions  = fi->isReadable();
             info.writePermissions = fi->isWritable();
             info.photoInfo        = pInfo;
@@ -227,7 +223,7 @@ bool UMSCamera::getThumbnail(const QString& folder, const QString& itemName, QIm
 
     // JPEG files: try to get thumbnail from Exif data.
 
-    DMetadata metadata(QFile::encodeName(folder + QString("/") + itemName));
+    DMetadata metadata(folder + QString("/") + itemName);
     thumbnail = metadata.getExifThumbnail(true);
     if (!thumbnail.isNull())
         return true;
@@ -259,91 +255,21 @@ bool UMSCamera::getThumbnail(const QString& folder, const QString& itemName, QIm
 
     // Finally, we trying to get thumbnail using DImg API (slow).
 
-    DImg dimgThumb(QFile::encodeName(folder + QString("/") + itemName));
+    DImg dimgThumb(folder + QString("/") + itemName);
 
     if (!dimgThumb.isNull())
     {
         thumbnail = dimgThumb.copyQImage();
         return true;
     }
-
-    if (loadKDEThumbCreator(folder, itemName, thumbnail))
-        return true;
-
     return false;
 }
 
-bool UMSCamera::loadKDEThumbCreator(const QString& folder, const QString& itemName, QImage& thumbnail)
-{
-    QString path     = folder + QString("/") + itemName;
-    QString mimeType = KMimeType::findByUrl(path)->name();
-
-    if (mimeType.isEmpty())
-    {
-        kDebug(50003) << "Mimetype not found" << endl;
-        return false;
-    }
-
-    QString mimeTypeAlt = mimeType.replace(QRegExp("/.*"), "/*");
-    QString plugin;
-
-    KService::List plugins = KServiceTypeTrader::self()->query("ThumbCreator");
-    for (KService::List::ConstIterator it = plugins.constBegin(); it != plugins.constEnd(); ++it)
-    {
-        QStringList mimeTypes = (*it)->property("MimeTypes").toStringList();
-        for (QStringList::ConstIterator mt = mimeTypes.constBegin(); mt != mimeTypes.constEnd(); ++mt)
-        {
-            if  ((*mt) == mimeType || (*mt) == mimeTypeAlt)
-            {
-                plugin = (*it)->library();
-                break;
-            }
-        }
-
-        if (!plugin.isEmpty())
-            break;
-    }
-
-    if (plugin.isEmpty())
-    {
-        kDebug(50003) << "No relevant plugin found " << endl;
-        return false;
-    }
-
-    KLibrary *library = KLibLoader::self()->library(QFile::encodeName(plugin));
-    if (!library)
-    {
-        kDebug(50003) << "Plugin library not found " << plugin << endl;
-        return false;
-    }
-
-    ThumbCreator *creator = 0;
-    newCreator create     = (newCreator)library->resolveSymbol("new_creator");
-    if (create)
-        creator = create();
-
-    if (!creator)
-    {
-        kDebug(50003) << "Cannot load ThumbCreator " << plugin << endl;
-        return false;
-    }
-
-    if (!creator->create(path, 256, 256, thumbnail))
-    {
-        kDebug(50003) << "Cannot create thumbnail for " << path << endl;
-        delete creator;
-        return false;
-    }
-
-    delete creator;
-    return true;
-}
-
 bool UMSCamera::getExif(const QString&, const QString&, char **, int&)
 {
-    // not necessary to implement this. read it directly from the file
+    // Not necessary to implement this. read data directly from the file
     // (done in camera controller)
-    kWarning(50003) << "Exif implemented yet in camera controller" << endl;
+    kWarning() << "Exif implemented yet in camera controller";
     return false;
 }
 
@@ -358,16 +284,16 @@ bool UMSCamera::downloadItem(const QString& folder, const QString& itemName, con
 
     if ( !sFile.open(QIODevice::ReadOnly) )
     {
-        kWarning(50003) << "Failed to open source file for reading: "
-                        << src << endl;
+        kWarning() << "Failed to open source file for reading: "
+                        << src;
         return false;
     }
 
     if ( !dFile.open(QIODevice::WriteOnly) )
     {
         sFile.close();
-        kWarning(50003) << "Failed to open dest file for writing: "
-                        << dest << endl;
+        kWarning() << "Failed to open dest file for writing: "
+                        << dest;
         return false;
     }
 
@@ -456,16 +382,16 @@ bool UMSCamera::uploadItem(const QString& folder, const QString& itemName, const
 
     if ( !sFile.open(QIODevice::ReadOnly) )
     {
-        kWarning(50003) << "Failed to open source file for reading: "
-                        << src << endl;
+        kWarning() << "Failed to open source file for reading: "
+                        << src;
         return false;
     }
 
     if ( !dFile.open(QIODevice::WriteOnly) )
     {
         sFile.close();
-        kWarning(50003) << "Failed to open dest file for writing: "
-                        << dest << endl;
+        kWarning() << "Failed to open dest file for writing: "
+                        << dest;
         return false;
     }
 
@@ -515,20 +441,17 @@ bool UMSCamera::uploadItem(const QString& folder, const QString& itemName, const
         dims  = meta.getImageDimensions();
         pInfo = meta.getPhotographInformation();
 
-        if (dt.isNull())
-        {
-            // If date is not found in metadata, use file time stamp
-            dt = fi.created();
-        }
+        if (dt.isNull()) // fall back to file system info
+            ImageScanner::creationDateFromFilesystem(fi);
 
         info.name             = fi.fileName();
-        info.folder           = !folder.endsWith("/") ? folder + QString("/") : folder;
+        info.folder           = !folder.endsWith('/') ? folder + QString("/") : folder;
         info.mime             = mime;
         info.mtime            = dt;
         info.size             = fi.size();
         info.width            = getImageDimensions ? dims.width()  : -1;
         info.height           = getImageDimensions ? dims.height() : -1;
-        info.downloaded       = GPItemInfo::DownloadUnknow;
+        info.downloaded       = GPItemInfo::DownloadUnknown;
         info.readPermissions  = fi.isReadable();
         info.writePermissions = fi.isWritable();
         info.photoInfo        = pInfo;
@@ -567,16 +490,18 @@ bool UMSCamera::cameraSummary(QString& summary)
     summary =  QString(i18n("<b>Mounted Camera</b> driver for USB/IEEE1394 mass storage cameras and "
                             "Flash disk card readers.<br/><br/>"));
 
+    // we do not expect titel/model/etc. to contain newlines,
+    // so we just escape HTML characters
     summary += i18n("Title: <b>%1</b><br/>"
                     "Model: <b>%2</b><br/>"
                     "Port: <b>%3</b><br/>"
                     "Path: <b>%4</b><br/>"
                     "UUID: <b>%5</b><br/><br/>",
-                    title(),
-                    model(),
-                    port(),
-                    path(),
-                    uuid());
+                    Qt::escape(title()),
+                    Qt::escape(model()),
+                    Qt::escape(port()),
+                    Qt::escape(path()),
+                    Qt::escape(uuid()));
 
     summary += i18n("Thumbnails: <b>%1</b><br/>"
                     "Capture image: <b>%2</b><br/>"
@@ -615,7 +540,7 @@ void UMSCamera::getUUIDFromSolid()
 {
     QList<Solid::Device> devices = Solid::Device::listFromType(Solid::DeviceInterface::StorageAccess);
 
-    foreach(const Solid::Device &accessDevice, devices)
+    foreach(const Solid::Device& accessDevice, devices)
     {
         // check for StorageAccess
         if (!accessDevice.is<Solid::StorageAccess>())
